class ClientError(SystemExit):

    def __init__(self,message):
        self.message = message
        super().__init__(self.message)

class ProxyError(ClientError):
    """Sorry, there was a problem with your request"""

class WrongAuthCredientals(ClientError):
    """Wrong username or password"""

class JsonDecodeError(ClientError):
    """Error in decoding json response"""

class LimitError(ClientError):
    """Requests limist reached"""

class FloodError(ClientError):
    """Flood control triggerred"""

class TokenError(ClientError):
    """Requests limist reached"""

class UnknownError(ClientError):
    """Unknown Error"""




from vkOsint.exceptions import WrongAuthCredientals,UnknownError

class Parsers:

    def __init__(self):
        super().__init__()

    def authDataParser(self,authData):
        if 'error_description' in authData and authData['error_description'] == 'Неправильный логин или пароль':
            raise WrongAuthCredientals(f"Login or password is incorrect\nLogin : {self.username}\nPassword : {self.password}\n")
        if 'access_token' and 'user_id' and 'secret' in authData:
            self.setTokens(authData)
            self.writeOutput('Authorized successfully')
        else:
            print(authData)
            raise UnknownError('Unknown Error')

    def updateTokensParser(self,tokensData):
        self.accessToken = tokensData['response']['token']
        self.secret = tokensData['response']['secret']

    def contactDataParser(self):
        if len(self.contactData['response']['found']):
            contact = self.contactData['response']['found'][0]['user']
            self.trackCode = contact['track_code']
            self.accessKey = contact['access_key']
            self.found = True
        else:
            self.found = False

    def profileParser(self):
        self.parsedData[self.phoneNumber] = self.profileData
        self.parsedData[self.phoneNumber]['found'] = self.found




from urllib.parse import urlparse


from vkOsint.mixins.api import Api
from vkOsint.mixins.utils import Utils
from vkOsint.mixins.parsers import Parsers


class vkOsint(Api,Utils,Parsers):

    def __init__(self,
                verbose: bool = True,
                proxy: str = None,
                sessionVerify: bool = True
                
    ):
        super().__init__()
        self.verbose = verbose
        self.setProxy(proxy,sessionVerify)
        
    def setProxy(self,proxy,sessionVerify):
        self.session.verify = sessionVerify
        if proxy:
            assert isinstance(proxy,str),f'Proxy must be string format, but now its {type(proxy)}'
            proxyHref = "{scheme}{href}".format(
                scheme="http://" if not urlparse(proxy).scheme else "",
                href=proxy,
            )
            self.session.proxies = {
                "http": proxyHref,
                "https": proxyHref,
            }


import random
import uuid
import hashlib

class Utils:
    def __init__(self):
        self.genUa()
        self.genAuthHeaders()
        self.genBaseHeaders()
        self.genDeviceUuid()
        super().__init__()

    def genUa(self):
        androidResolutions = ['1024x600', '1280x800', '480x854', '720x1280', '1200x1290', '2560x1600', '768x1280', '1080x1920','800x1280']
        androidVersions = [{'sdkVersion':'28','androidVersion':'9'},{'sdkVersion':'29','androidVersion':'10'},{'sdkVersion':'30','androidVersion':'11'},{'sdkVersion':'31','androidVersion':'12'},{'sdkVersion':'33','androidVersion':'13'}]
        androidModels = ['SM-A505F','SM-A515','SM-A516F','SM-A526B','SM-A710F','SM-A530F']
        sdkV,androidV = random.choice(androidVersions).values()
        self.useragent = f'VKAndroidApp/7.24-3439 (Android {androidV}; SDK {sdkV}; armeabi-v7a; samsung {random.choice(androidModels)}; ru; {random.choice(androidResolutions)})'

    def genDeviceUuid(self):
        self.deviceUuid = str(uuid.uuid4())
        
    def genDeviceHash(self):
        seed = hashlib.md5(self.username.encode('utf-8') + self.password.encode('utf-8')).hexdigest()
        m = hashlib.md5()
        m.update(seed.encode('utf-8'))
        self.deviceHash = m.hexdigest()[:16]

    def genAuthHeaders(self):
        self.authHeaders = {
            'Host': 'oauth.vk.com',
            'User-Agent': self.useragent
        }

    def genBaseHeaders(self):
        self.baseHeaders = {
            'Host': 'api.vk.com',
            'Cache-Control': 'no-cache',
            'User-Agent': self.useragent,
            'X-Vk-Android-Client': 'new',
            'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8'
        }

    def writeOutput(self,message):
        if self.verbose:
            print(message)

    def changeProxy(self,proxy: str,sessionVerify: bool = True):
        self.setProxy(proxy,sessionVerify)

    def setTokens(self,authData):
        self.accessToken = authData.get('access_token',None)
        self.secret =  authData.get('secret',None)
        self.userId =  authData.get('user_id',None)
        self.webviewRefreshToken = authData.get('webview_refresh_token',None)
        self.webviewAccessToken = authData.get('webview_access_token',None)

    def getTokens(self):
        return {
            'accessToken':self.accessToken,
            'secret':self.secret,
            'userId':self.userId,
            'webviewRefreshToken':self.webviewRefreshToken,
            'webviewAccessToken':self.webviewAccessToken

        }
        






from setuptools import find_packages, setup

longDescription = """
## vkOsint - Python package for searching vk accounts by phone number

Features:
* Authorization by login and password
* Tokens parsing
* Accounts searching by phone number
* Source code contains all the api's hash functionality

Package uses Private api without selenium.
Developed by reverse engineering & api sniffing

Support **Python >= 3.10**

## ToDo
- [ ] 2fa authorization
- [ ] Osint function return value filters
## Installation
```console
$ pip install vkOsint
```
### Basic Usage

``` python
import json

from vkOsint import vkOsint

vkInstance = vkOsint(proxy='[proxy]')
vkInstance.login(username='[login]',password='[password]')
parsedData = vkInstance.osint(phoneNumbers=['+7..........','+7..........'])
with open('results.json','w') as file:
    json.dump(parsedData,file,indent=4,ensure_ascii=True)
```
"""

requirements = [
    "PySocks==1.7.1",
    "requests==2.28.2",
    "urllib3==1.26.14",
]

setup(
    name="vkOsint",
    version="1.0",
    author="DIJIRO",
    author_email="de.sm1rnov@yandex.ru",
    license="MIT",
    url="https://github.com/DIJIRO/vkOsint",
    install_requires=requirements,
    keywords=[
        "vk private api",
        "vk osint",
        "vkOsint",
        "vk-api"
    ],
    description="Python package for searching vk accounts by phone number",
    long_description=longDescription,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    python_requires=">=3.10",
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



import requests
import json
import urllib
import hashlib
from urllib3.exceptions import MaxRetryError,NewConnectionError
from socket import gaierror

from vkOsint.exceptions import *

class Api:
    def __init__(self):
        self.session = requests.session()
        super().__init__()
    
    def login(self,
              username: str,
              password: str
    ):
        self.username = username
        self.password = password
        self.genDeviceHash()
        authParams = {
            'scope': 'nohttps,all',
            'client_id': '2274003',
            'client_secret': 'hHbZxrka2uZ6jB1inYsH',  # apps secret token - static
            'v': '5.96',
            'lang': 'ru',
            '2fa_supported': '1',
            'lang': 'ru',
            'device_id': self.deviceHash,
            'grant_type': 'password',
            'username': self.username,
            'password': self.password,
            'libverify_support': '0'
        }
        try:
            authData = self.session.get('https://oauth.vk.com/token', params=authParams, headers=self.authHeaders).json()
            self.authDataParser(authData)
        except json.decoder.JSONDecodeError:
            raise JsonDecodeError("Error in decoding response json")
        except (requests.exceptions.ProxyError,MaxRetryError,NewConnectionError,gaierror):
            raise ProxyError("Lost connection to server, Inavild proxy")

    def tokenAuth(self,
                  username: str,
                  accessToken: str,
                  secret:str,
                  userId: str
    ):
        self.username = username
        self.setTokens({'access_token':accessToken,'secret':secret,'user_id':userId})

    def updateTokens(self):
        data = {'v': '5.96',
                'https': '1',
                'timestamp': '0',
                'receipt2': 'eyJhbGciOiAibm9uZSJ9.eyJub25jZSI6ICJ0ZXN0PT0ifQ.',
                'device_id': self.deviceUuid,
                'receipt': 'yssp9o9p9pamz5t-nvmq8spgwtin3e0==',
                'lang': 'ru',
                'access_token': self.accessToken
                }
        tokensData = self.makeRequest('auth.refreshToken',data)
        self.updateTokensParser(tokensData)

    def osint(self,phoneNumbers: list):
        assert isinstance(phoneNumbers, list), f'Phone numbers must be list format, but now its {type(phoneNumbers)}, you can also provide 1 number in the list'
        self.phoneNumbers = phoneNumbers
        self.parsedData = {}
        for self.phoneNumber in phoneNumbers:
            self.writeOutput(f'Searching for {self.phoneNumber}')
            self.profileData = {}  
            self.searchContact()
            self.contactDataParser()
            if self.found:
                self.getProfile()
            self.profileParser()
        return self.parsedData

    def searchContact(self):
        contacts = {
            "phone":
                {
                    "user_contact": self.username,
                    "contacts": self.phoneNumber
                }
        }
        data = {'v': '5.96',
                'https': '1',
                'device_id': self.deviceUuid,
                'fields': 'online,photo_50,photo_100,photo_200,career,city,country,education,verified,trending',
                'lang': 'ru',
                'search_only': '0',
                'count': '5000',
                'contacts': json.dumps(contacts, separators=(',', ':')),
                'need_mutual': '1',
                'access_token': self.accessToken,
                }
        contactData = self.makeRequest('account.searchContacts', data)
        self.contactData = contactData

    def getProfile(self):
        data = {
            'v': '5.96',
            'https': '1',
            'track_code': self.trackCode,
            'source': 'friends_import_address_book',
            'gift_count': '25',
            'track_events': '1',
            'skip_hidden': '1',
            'photo_sizes': '1',
            'func_v': '8',
            'access_keys':  self.accessKey,
            'device_id': self.deviceUuid,
            'photo_count': '25',
            'lang': 'ru',
            'ref': 'profile',
            'user_id': self.userId,
            'access_token': self.accessToken

        }
        self.profileData = self.makeRequest('execute.getFullProfileNewNew', data)

    def makeRequest(self,method,data):
        baseUrl = 'https://api.vk.com/method/'
        sigStr = f'/method/{method}?' + urllib.parse.unquote(urllib.parse.urlencode(data)) + self.secret
        sig = hashlib.md5(sigStr.encode()).hexdigest() #Signature generate
        data['sig'] = sig
        try:
            response = self.session.post(f'https://api.vk.com/method/{method}', headers=self.baseHeaders, data=data)
            jsonResp = response.json()
            if 'error' in jsonResp:
                if jsonResp['error']['error_code'] == 13:
                    raise LimitError("Requests limist reached")
                elif jsonResp['error']['error_msg'] == 'Flood control':
                    raise FloodError("Flood control triggerred, change bot account")
                elif jsonResp['error']['error_msg'] == 'Token confirmation required':
                    raise TokenError("Access & Secret tokens expired")
            return jsonResp
        except json.decoder.JSONDecodeError:
            raise JsonDecodeError("Error in decoding response json")
        except (requests.exceptions.ProxyError,MaxRetryError,NewConnectionError,gaierror):
            raise ProxyError("Lost connection to server, Inavild proxy")

